COMMENT \

   *************************************************************************
   *                       XDLE - Dos Line Editor                          *
   *                 (c) 1990-1992 by Alessandro Scotti                    *
   *                            Version 1.5                                *
   *************************************************************************

   Ultima revisione: giugno 1992

\

TITLE "XDLE 1.5"

.MODEL TINY
.CODE
.186

MASM
LOCALS

;
; Macro ed EQU
;

MsDos MACRO Function                        ; Funzioni DOS
  IF Function LT 0FFh
    mov  ah, Function
    int  21h
  ELSE
    mov  ax, Function
    int  21h
  ENDIF
ENDM

Video MACRO Function                        ; Funzioni BIOS per il video
  mov  ah, Function
  int  10h
ENDM

TRUE       EQU  1                           ; Variabili booleane
FALSE      EQU  0                           ;
EOS        EQU  13                          ; Identificatore di fine stringa
TAB        EQU  9                           ; ASCII Tab
LF         EQU  10                          ; ASCII Line-Feed
CR         EQU  13                          ; ASCII Carriage-Return
SPACE      EQU  32                          ; ASCII Spazio

CANF3      EQU   1                          ; Flags
SAVELINE   EQU   2                          ;
ACTIVE     EQU   4                          ;
INSERT     EQU   8                          ;
PROMPT     EQU  16                          ;
RESETREAD  EQU  32                          ;
OVERWRITE  EQU  64                          ;
NOCHGCRS   EQU  128                         ;

IFDEF NO186                                 ; Per compilare senza
  IS8086  EQU TRUE                          ; istruzioni estese
ELSE                                        ; Es.: tasm xdle /dNO186
  IF (@Cpu AND 1) EQ 1                      ; 80186 (o superiore)
    DISPLAY "Using extended instructions (80186/286/386)"
    IS8086  EQU FALSE
  ELSE
    IS8086  EQU TRUE
  ENDIF
ENDIF

org 100h                                    ; Inizio files .COM
Entry:     jmp  Start

;  Stack e variabili di sistema
;
StackBot   DB   CR,'   ',CR,LF
           DB   'XDLE 1.5 - (c) 1990-92 Alessandro Scotti',CR,LF,1Ah
;
;  Routine di istallazione (usa e getta!!!)
;
ExitTSR:   mov  dx, OFFSET PromptMessage
           call Print
           mov  dx, OFFSET EndOfTSR
           mov  [BuffStart], dx
           add  dx, [BufferSize]
           mov  [BuffEnd], dx
           call BufferInitAll               ; Inizializza il buffer
           add  dx, 0Fh
         IF IS8086
           mov  cl, 4
           shr  dx, cl                      ; Trasforma in paragrafi
         ELSE
           shr  dx, 4
         ENDIF
           MsDos 3100h                      ; TSR (exit code 0)
;
           DB   128-($-StackBot) dup (?)    ; Stack di 128 bytes (in caso di
                                            ; necessita' sconfinerebbe nella
                                            ; zona parametri del PSP, che non
                                            ; viene utilizzata)
StackTop   DW   ?
NewSS      DW   ?                           ; Copia del Code Segment
SavedSS    DW   ?                           ; Copia stack segment
SavedSP    DW   ?                           ; Copia stack pointer
OldDosInt  LABEL DWORD                      ; Vecchio interrupt 21h
OldDosIntO DW   ?
OldDosIntS DW   ?

;  Flags e opzioni
;
TabSize    DW   8                           ; Dimensione TAB
MinLength  DW   3                           ; Lungh. minima stringa da salvare
Flags      DB   ?

;  Variabili video
;
FirstChar  DB   ?                           ; Primo carattere buffer precedente
VideoPage  DB   ?                           ; Pagina video
VideoCol   DB   ?                           ; Numero di colonne del video
CrsSize    DW   ?
NewCrsSize DW   ?
CrsSwap    DW   2000h                       ; Valori per spengere il cursore
CursorXY   DW   ?                           ; Coordinate del cursore
PagePos    DW   ?                           ; Posizione relativa stringa-schermo
XPos       DB   ?                           ; Posizione del cursore
VdWid      DB   ?                           ; Ampiezza schermo disponibile

;  Variabili editor e buffer comandi
KeyCode    DW   ?                           ; Codice del carattere letto
StrIndex   DW   ?                           ; Indice carattere al cursore
LastRead   DW   ?
WritePtr   DW   ?
ReadPtr    DW   ?
BuffStart  DW   ?
BuffEnd    DW   ?

IDCode     DB   3    dup(0)                 ; Per identificare il DLE residente

;
; Editor di linea
;

;  Inserisce il carattere AL nella posizione BX della stringa StrBuffer
InsChar PROC NEAR
           push si
           add  si, bx
           dec  si
           mov  di, si
           mov  ah, al                      ; AH = carattere da inserire
@@Loop:    SEGES
           lodsb
           xchg al, ah
           stosb
           cmp  al, EOS
           jne  @@Loop                      ; Sposta i caratteri seguenti
           pop  si
;  Subroutine!
SetSaveFlag:
           or   [Flags], SAVELINE+CANF3
           ret
InsChar ENDP

;  Cancella il carattere numero BX dalla stringa StrBuffer
DelChar PROC NEAR
           push si
           add  si, bx
           mov  di, si
           dec  di
@@Loop:    SEGES
           lodsb
           stosb
           cmp  al, EOS
           jne  @@Loop
           pop  si
           jmp  SetSaveFlag
DelChar ENDP

;  Ritorna in AL il carattere numero BX della stringa StrBuffer
GetIndexChar PROC NEAR
           mov  al, es:[bx+si-1]
           ret
GetIndexChar ENDP

;  Ritorna in CX la lunghezza della stringa StrBuffer (non modifica SI)
StrLen PROC NEAR
           push si
           xor  cx, cx
@@Loop:    inc  cx
           SEGES
           lodsb
           cmp  al, EOS
           jne  @@Loop
           dec  cx
           pop  si
           ret
StrLen ENDP

;  Ritorna in BX la posizione del cursore all'interno della stringa
Index PROC NEAR
           mov  bx, [StrIndex]
           ret
Index ENDP

;  Ritorna in CX la colonna su cui si trova il cursore
GetCursorCol PROC NEAR
           xor  cx, cx
           mov  bx, 1                       ; BX = indice
@@Loop:    inc  cx
           cmp  bx, [StrIndex]              ; Siamo arrivati al cursore?
           jae  @@Exit                      ; Si, esci
           call GetIndexChar                ; Leggi il carattere al cursore
           inc  bx
           cmp  al, TAB                     ; E' un TAB?
           jne  @@Loop                      ; No, continua
@@TabLoop: mov  ax, cx                      ; Si, espandi il TAB
           cwd
           div  [TabSize]
           or   dx, dx
           jz   @@Loop
           inc  cx
           jmp  @@TabLoop
@@Exit:    ret
GetCursorCol ENDP

;  Muove il cursore a destra
GoRight PROC NEAR
           push ax                          ; Salva AX
           call GetCursorCol                ; CX = colonna del cursore
           push cx                          ; Salva CX
           inc  [StrIndex]                  ; Cursore a destra
           call GetCursorCol                ; CX = nuova colonna del cursore
           pop  ax                          ; CX = differenza tra la posizione
           sub  cx, ax                      ;      nuova e quella vecchia
@@Loop:    mov  al, [XPos]
           cmp  al, [VdWid]                 ; Il curs. video e' a fine schermo?
           jb   @@IncXPos                   ; No, incrementa
           inc  [PagePos]                   ; Si, sposta l'intera finestra
           jmp  SHORT @@LoopTest
@@IncXPos: inc  [XPos]
@@LoopTest:loop @@Loop                      ; Ripeti CX volte
           pop  ax                          ; Recupera AX
           ret
GoRight ENDP

;  Muove il cursore a sinistra
GoLeft PROC NEAR
           push ax                          ; Salva AX
           call GetCursorCol                ; CX = colonna del cursore
           push cx                          ; Salva CX
           dec  [StrIndex]                  ; Cursore a sinistra
           call GetCursorCol                ; CX = nuova colonna del cursore
           mov  ax, cx
           pop  cx                          ; CX = differenza tra la posizione
           sub  cx, ax                      ;      vecchia e quella nuova
@@Loop:    dec  [XPos]                      ; Cursore video a sinistra
           jnz  @@LoopTest
           inc  [XPos]
           dec  [PagePos]                   ; Sposta la finestra
@@LoopTest:loop @@Loop
           pop  ax                          ; Recupera AX
           ret
GoLeft ENDP

;  Pulisce la riga video corrente dal cursore in poi
ClrEol PROC NEAR
           mov  bh, [VideoPage]
           mov  cl, [VideoCol]
           mov  ch, 0                       ; CX = numero di colonne video
           push cx
           Video 03h                        ; Read Cursor Position
           pop  cx
           sub  cl, dl                      ; DL = colonna attuale
           mov  ax, 0A20h
           int  10h                         ; Stampa CX spazi
           ret
ClrEol ENDP

;  Accende/spenge il cursore
SwapCursor PROC NEAR
           mov  bh, [VideoPage]
           Video 03h
           push cx
           mov cx, [CrsSwap]
           Video 01h
           pop cx
           mov [CrsSwap], cx
           ret
SwapCursor ENDP

;  Controlla se Index > 1
TestIndexGT1 PROC NEAR
           call Index                       ; Indice in BX
           cmp  bx, 1
           ret
TestIndexGT1 ENDP

;  Controlla se Index > StrLen
TestIndexGTStrLen PROC NEAR
           call Index                       ; Indice in BX
           call StrLen                      ; Lunghezza in CX
           cmp  bx, cx                      ; Esegue il confronto
           ret
TestIndexGTStrLen ENDP

;  Controlla se il carattere indirizzato da BX e' un blank
TestCharBlank PROC NEAR
           call GetIndexChar                ; AL = carattere da controllare
           cmp  al, SPACE                   ; E' uno spazio?
           je   @@Exit                      ; Si, esci
           cmp  al, TAB                     ; No, confronta con TAB...
@@Exit:    ret                              ; ... ed esci
TestCharBlank ENDP

;  Controlla se il carattere indirizzato da BX e' una lettera valida
TestCharLetter PROC NEAR
           call GetIndexChar
           cmp  al, '0'
           jb   @@ExitNo
           cmp  al, '9'
           jbe  @@ExitOk
           cmp  al, 'A'
           jb   @@ExitNo
           cmp  al, 'Z'
           jbe  @@ExitOk
           cmp  al, '_'
           je   @@ExitOk
           cmp  al, 'a'
           jb   @@ExitNo
           cmp  al, 'z'
           jbe  @@ExitOk
@@ExitNo:  stc
           ret
@@ExitOk:  clc
           ret
TestCharLetter ENDP

;  Destra
RightKey PROC NEAR
           call TestIndexGTStrLen           ; Il cursore e' gia' alla fine?
           ja   @@Exit                      ; Si, esci
           call GoRight                     ; Vai a destra
@@Exit:    ret
RightKey ENDP

;  Sinistra
LeftKey PROC NEAR
           call TestIndexGT1                ; Il cursore e' gia' all'inizio?
           jbe  @@Exit                      ; Si, esci
           call GoLeft                      ; Vai a sinistra
@@Exit:    ret
LeftKey ENDP

;  Fine riga
EndKey PROC NEAR
@@Loop:    call TestIndexGTStrLen           ; Il cursore e' a fine stringa?
           ja   @@Exit                      ; Si, esci
           call GoRight                     ; Vai a destra
           jmp  @@Loop                      ; Ripeti il ciclo
@@Exit:    ret
EndKey ENDP

;  Inizio riga
HomeKey PROC NEAR
@@Loop:    call TestIndexGT1                ; Il cursore e' all'inizio?
           jbe  @@Exit                      ; Si, esci
           call GoLeft                      ; Vai a sinistra
           jmp  @@Loop                      ; Ripeti il ciclo
@@Exit:    ret
HomeKey ENDP

;  Insert on/off
InsKey PROC NEAR
           xor  [Flags], INSERT             ; Commuta il flag
           test [Flags], NOCHGCRS
           jnz  @@Exit
           mov  cx, [NewCrsSize]
           test [Flags], INSERT
           jnz  @@Cont
           sub  ch, 2                       ; Cursore in modo inserimento
@@Cont:    push si
           Video 01h                        ; Set Cursor Size
           pop  si
@@Exit:    ret
InsKey ENDP

;  Parola a destra
CtrlRight PROC NEAR
@@While1:  call TestWordRight
           jc   @@While2                    ; Si, passa alla seconda parte
           call GoRight                     ; Vai a destra
           jmp  @@While1                    ; Ripeti il primo ciclo
@@While2:  call TestWordRight
           jnc  @@Exit                      ; No, esci
           call GoRight                     ; Vai a destra
           jmp  @@While2                    ; Ripeti il secondo ciclo
@@Exit:    ret
;  Subroutine! In caso l'indice sia a fine linea provoca l'uscita della
;  funzione chiamante
TestWordRight:
           call TestIndexGTStrLen
           ja   @@Abort
           jmp  TestCharLetter
@@Abort:   pop  bx                          ; Toglie l'IP dallo stack
           ret
CtrlRight ENDP

;  Cancella il carattere al cursore
DelKey PROC NEAR
           call TestIndexGTStrLen           ; Fine stringa?
           ja   @@Exit                      ; Si, esci
           call DelChar                     ; Cancella il carattere
@@Exit:    ret
DelKey ENDP

;  Parola a sinistra
CtrlLeft PROC NEAR
@@While1:  call @@Sub
           jnc  @@While2                    ; No, passa al secondo ciclo
           call GoLeft                      ; Si, vai a sinistra...
           jmp  @@While1                    ; ... e continua
@@While2:  call @@Sub
           jc   @@Exit                      ; No, esci
           call GoLeft                      ; Si, vai a sinistra...
           jmp  @@While2                    ; ... e continua
@@Exit:    ret
@@Sub:     call TestIndexGT1
           jbe  @@Abort
           dec  bx
           jmp  TestCharLetter
@@Abort:   pop  bx
           ret
CtrlLeft ENDP

;  Cancella tutto a sinistra del cursore
CtrlHome PROC NEAR
           call Index                       ; Indice in BX
           mov  cx, bx
           dec  cx                          ; CX = indice - 1
           jcxz ResetIndex                  ; Esci se inizio stringa
@@Loop:    mov  bx, 1
           call GetIndexChar                ; Legge il primo carattere
           cmp  al, EOS                     ; E' il carattere di fine stringa?
           je   ResetIndex                  ; Si, esci
           call DelChar                     ; No, cancella
           loop @@Loop                      ; Ripeti CX volte
;  Subroutine!
ResetIndex:mov  [XPos], 1
           mov  [PagePos], 0
           mov  [StrIndex], 1
           ret
CtrlHome ENDP

;  Cancella tutto dal cursore in poi
CtrlEnd PROC NEAR
@@Loop:    call Index                       ; Indice in BX
           call GetIndexChar                ; AL = carattere al cursore
           cmp  al, EOS                     ; E' il carattere di fine stringa?
           je   @@Exit                      ; Si, esci
           call DelChar                     ; No, cancella
           jmp  @@Loop                      ; Ripeti fino a EOS
@@Exit:    ret
CtrlEnd ENDP

;  Recupera l'ultimo comando inserito
F3Key PROC NEAR
           test [Flags], CANF3
           jnz  @@Exit
           mov  al, [FirstChar]
           mov  es:[si], al
           call InitStrPos
@@Exit:    ret
F3Key ENDP

;  Recupera il precedente comando nel buffer
;
UpKey PROC NEAR
           test [Flags], PROMPT
           jz   @@Exit
           call TestEmptyBuffer
           call BufferGetPrev
           call InitStrPos
;  Subroutine!
ResetSaveFlag:
           and  [Flags], NOT SAVELINE
           or   [Flags], CANF3
@@Exit:    ret
UpKey ENDP

;  Recupera il successivo comando nel buffer
DownKey PROC NEAR
           test [Flags], PROMPT
           jz   @@Exit
           call TestEmptyBuffer
           call BufferGetNext
           call InitStrPos
           call ResetSaveFlag
@@Exit:    ret
DownKey ENDP

;  Cancella il carattere a sinistra del cursore
BackSpace PROC NEAR
           call TestIndexGT1                ; Cursore a inizio stringa?
           jbe  @@Exit                      ; Si, esci
           call GoLeft                      ; Sinistra
           call Index
           call DelChar                     ; Cancella il carattere al cursore
@@Exit:    ret
BackSpace ENDP

;  Cancella la parola a destra del cursore
DeleteWord PROC NEAR
           call TestWordRight   ; Cancella il primo carattere
           call DelChar
@@While1:  call TestWordRight   ; Cancella le lettere
           jc   @@While2
           call DelChar
           jmp  @@WHile1
@@While2:  call TestWordRight   ; Cancella i "blank"
           call TestCharBlank
           jne  @@Exit
           call DelChar
           jmp  @@While2
@@Exit:    ret
DeleteWord ENDP

;  Cancella l'intera linea
DeleteLine PROC NEAR
           mov  BYTE PTR es:[si], EOS
           mov  [LastRead], 0
           jmp  SHORT InitStrPos
DeleteLine ENDP

;  Inizializza XPos e PagePos
InitStrPos PROC NEAR
           call ResetIndex                  ; Definita nella proc CtrlHome
           jmp  EndKey
InitStrPos ENDP

;  Esegue il comando senza memorizzare la linea
Execute PROC NEAR
           mov  [KeyCode], 000Dh            ; Forza il CR
           jmp  ResetSaveFlag
Execute ENDP

; Seleziona l'ultimo comando memorizzato
LastCommand PROC NEAR
        call    BufferResetReadPtr
        call    UpKey
        ret
LastCommand ENDP

;  Legge un tasto: AL = ASCII, AH = 1 se codice esteso, altrimenti AH = 0
ReadKey PROC NEAR
@@Wait:
           mov  ah, 01h
           int  16h
           jz   @@Wait
           MsDos 08h                        ; DOS ReadKey
           and  ax, 00FFh                   ; Azzera AH e controlla se AL e' 0
           jnz  @@Exit                      ;
           MsDos 08h                        ; Leggi il codice esteso
           mov  ah, 1
@@Exit:    ret
ReadKey ENDP

;  Stampa la stringa in fase di editazione
UpdateDisplay PROC NEAR
           push si
           call SwapCursor                  ; Spenge il cursore
           mov  dx, [CursorXY]
           mov  bh, [VideoPage]
           Video 02h                        ; Aggiusta il cursore
           call StrLen                      ; CX = lunghezza stringa
           xor  bx, bx                      ; Indice stringa
           xor  di, di                      ; Contatore caratteri stampati
           mov  dl, [VdWid]
           mov  dh, 0
           add  dx, [PagePos]
@@Loop:    inc  bx
           cmp  bx, cx                      ; Fine stringa?
           ja   @@Exit                      ; Si, esci
           call GetIndexChar                ; AL = carattere al cursore
           cmp  al, TAB                     ; E' un TAB?
           je   @@TabLoop                   ; Si, espandi il TAB
           call @@PutChar                   ; Stampa il carattere
           jmp  @@Loop                      ; Ripeti il ciclo principale
@@TabLoop: mov  al, SPACE
           call @@PutChar                   ; Stampa uno spazio
           mov  ax, di
           push dx
           cwd
           div  [TabSize]
           or   dx, dx
           pop  dx
           jnz  @@TabLoop                   ; Ciclo espansione TAB
           jmp  @@Loop                      ; Ciclo principale
@@Exit:    call ClrEol                      ; Pulisce la riga
           mov  dx, [CursorXY]
           add  dl, [XPos]
           dec  dl
           Video 02h                        ; Aggiusta il cursore
           call SwapCursor                  ; Accende il cursore
           pop  si
           ret
;  Stampa 'virtualmente' un carattere
@@PutChar: inc  di                          ; Contatore caratteri
           cmp  di, [PagePos]
           jbe  @@PCExit                    ; Fuori schermo (a sinistra), esci
           cmp  di, dx
           ja   @@PCExit                    ; Fuori schermo (a destra), esci
           push bx
           mov  bh, [VideoPage]
           xor  bl, bl
           Video 0Eh                        ; Write Char TTY
           pop  bx
@@PCExit:  ret
UpdateDisplay ENDP

;  Elimina i blanks iniziali dalla stringa editata
RemoveBlanks PROC NEAR
@@Loop:    call StrLen
           jcxz @@Exit
           mov  bx, 1
           call TestCharBlank
           jne  @@Exit
           call DelChar
           jmp  @@Loop
@@Exit:    ret
RemoveBlanks ENDP

TSRTestStart   label byte
;  Mini-editor di linea
LineEdit PROC NEAR
           push si
           and  [Flags], NOT (SAVELINE+CANF3+INSERT)
           Video 0Fh                        ; Get Current Video Mode
           mov  [VideoPage], bh             ; Salva la pagina attiva...
           mov  [VideoCol], ah              ; ... e il numero di colonne
@@Lab1:    Video 03h                        ; Read Cursor Position and Size
           mov  [CursorXY], dx              ; Salva la posizione del cursore
           mov  [CrsSize], cx               ; Salva la dimensione del cursore
           mov  [NewCrsSize], cx
           mov  al, [VideoCol]
           sub  al, dl
           sub  al, 2
           mov  [VdWid], al                 ; Numero di colonne disponibili
           pop  si
           test [Flags], OVERWRITE
           jnz  @@Lab2
           or   [Flags], INSERT
@@Lab2:    call InsKey
           call BufferResetReadPtr
           call DeleteLine
@@Repeat:  call UpdateDisplay
           call ReadKey                     ; Legge un tasto...
           mov  [KeyCode], ax               ; ... e lo salva
;  Esamina la tavola dei comandi
           mov  bx, OFFSET CmdTable         ; Tavola comandi
           sub  bx, 4
@@SpLoop:  add  bx, 4
           cmp  WORD PTR [bx], 0            ; Fine tavola?
           je   @@CheckAlias                ; S, controlla gli alias
           cmp  [bx], ax                    ; E' il codice giusto?
           jne  @@SpLoop                    ; No, cerca ancora
           inc  bx
           inc  bx                          ; BX -> indirizzo della procedura
           call [bx]                        ; Esegui la procedura
           jmp  SHORT @@Until               ; E ripeti il ciclo principale
;  Controlla se il codice e' un ASCII valido e lo inserisce nella stringa
@@CheckAlias:
           mov  bx, OFFSET AliasTable       ; Tavola alias
           sub  bx, 4
@@AliasLoop:
           add  bx, 4
           cmp  WORD PTR [bx], 0            ; Fine tavola?
           je   @@NoMatch                   ; S, controlla come ASCII
           cmp  [bx], ax                    ; Codice giusto?
           jne  @@AliasLoop                 ; No, cerca ancora
           call ExecAlias
           jmp  SHORT @@Until
@@NoMatch:
           cmp  ah, 1
           je   @@Until                     ; Codice esteso non valido
           cmp  al, TAB                     ; E' un TAB?
           je   @@Ok                        ; Si, continua
           cmp  al, SPACE
           jb   @@Until                     ; Codice non valido
@@Ok:      test [Flags], INSERT             ; Siamo in modo inserimento?
           jz   @@Insert                    ; Si, esegui
           call TestIndexGTStrLen           ; Il cursore e' oltre fine stringa?
           ja   @@Insert
           call DelChar                     ; Modo overwrite
@@Insert:  call StrLen                      ; CX = lunghezza stringa
           cmp  cl, es:[si-2]               ; Stringa troppo lunga?
           jae  @@Until                     ; Si, tralascia il comando
           mov  ax, [KeyCode]               ; AL = codice carattere
           call Index
           call InsChar                     ; Inserisce il carattere
           call GoRight                     ; Sposta a destra il cursore
@@Until:   cmp  [KeyCode], 000Dh            ; E' stato premuto (o simulato) CR?
           jne  @@Repeat                    ; No, continua il ciclo
           call StrLen                      ; CX = lunghezza stringa
           mov  es:[si-1], cl
           test [Flags], SAVELINE           ; Bisogna salvare la linea?
           jz   @@Exit                      ; No, esci
           test [Flags], PROMPT             ; Siamo al prompt?
           jz   @@Exit                      ; No, esci
           call RemoveBlanks                ; Elimina gli spazi iniziali
           cmp  cx, [MinLength]             ; Ok con la lunghezza?
           jb   @@Exit                      ; No, stringa troppo corta: esci
           call BufferPutString             ; Salva la stringa appena editata
@@Exit:    mov  cx, [CrsSize]
           Video 01h
           ret                              ; Fine procedura!
LineEdit ENDP
TSRTestEnd     label byte

;  Tabella dei comandi riconosciuti e indirizzi delle rispettive procedure
;
CmdTable   DW   0147h, Homekey              ;
           DW   014Bh, LeftKey              ;
           DW   014Dh, RightKey             ;
           DW   014Fh, EndKey               ;
           DW   0148h, UpKey                ;
           DW   0150h, DownKey              ;
           DW   0152h, InsKey               ;
           DW   0153h, DelKey               ;
           DW   0173h, CtrlLeft             ;
           DW   0174h, CtrlRight            ;
           DW   0175h, CtrlEnd              ;
           DW   0176h, BufferDelEntry       ; Ctrl - PgDn
           DW   0177h, CtrlHome             ;
           DW   0184h, BufferInitAll        ; Ctrl - PgUp
           DW   013Dh, F3Key                ;
           DW   0008h, BackSpace            ;
           DW   0005h, Execute              ; Ctrl - E
           DW   000Ch, LastCommand          ; Ctrl - L
           DW   0014h, DeleteWord           ; Ctrl - T
           DW   001Bh, DeleteLine           ; Escape
           DW   0h                          ; Fine tavola

;  Tabella degli alias (macro) riconosciuti e rispettivi tasti
;
EXECNOW EQU     '@'

;  Comandi DOS pi usati
;
Alt_A   DB      ' a:', EOS
Alt_B   DB      ' b:', EOS
Alt_C   DB      EXECNOW, 'cls', EOS
Alt_D   DB      'dir ', EOS
Alt_E   DB      'erase ', EOS
Alt_F   DB      'format ', EOS
Alt_G   DB      'debug ', EOS
Alt_M   DB      'mode ', EOS
Alt_Q   DB      EXECNOW, 'exit', EOS
Alt_R   DB      'rename ', EOS
Alt_S   DB      '*.*', EOS      ; "Star"
Alt_X   DB      'xcopy ', EOS

AliasTable      LABEL   WORD
        DW      011Eh,  Alt_A
        DW      0130h,  Alt_B
        DW      012Eh,  Alt_C
        DW      0120h,  Alt_D
        DW      0112h,  Alt_E
        DW      0121h,  Alt_F
        DW      0122h,  Alt_G
        DW      0132h,  Alt_M
        DW      0110h,  Alt_Q
        DW      0113h,  Alt_R
        DW      011Fh,  Alt_S
        DW      012Dh,  Alt_X
        DW      0h

; Procedura di gestione degli alias
ExecAlias       PROC NEAR
        push    si
        inc     bx
        inc     bx
        mov     di, [bx]
        push    di
        call    StrLen
        or      cx, cx
        jnz     @@DontReset
        call    ResetSaveFlag
@@DontReset:
        add     si, cx
        pop     di
        xchg    di, si
        cmp     BYTE PTR [si], EXECNOW
        jne     @@NoExec
        mov     [KeyCode], 000Dh
        inc     si
@@NoExec:
        cld
@@Loop:
        lodsb
        stosb
        cmp     al, EOS
        jne     @@Loop
        pop     si
        call    InitStrPos
        cmp     [KeyCode], 000Dh
        jne     @@Exit
        call    UpDateDisplay
@@Exit:
        ret
ExecAlias       ENDP

;
; Buffer linee editate
;

;  Controlla se il buffer e' vuoto e in caso affermativo
;  esce dalla procedura chiamante
TestEmptyBuffer PROC NEAR
           mov  di, [ReadPtr]
           cmp  BYTE PTR [di], EOS
           jne  @@Exit
           pop  ax                          ; Toglie dallo stack questa chiamata
@@Exit:    ret
TestEmptyBuffer ENDP

;  Sposta DI alla precedente locazione nel buffer comandi
BufferPrev PROC NEAR
           cmp  di, [BuffStart]
           ja   @@Continue
           mov  di, [BuffEnd]
@@Continue:dec  di
           cmp  BYTE PTR [di], EOS
           ret
BufferPrev ENDP

;  Sposta DI alla successiva locazione nel buffer comandi
BufferNext PROC NEAR
           inc  di
           cmp  di, [BuffEnd]
           jb   @@Exit
           mov  di, [BuffStart]
@@Exit:    cmp  BYTE PTR [di], EOS
           ret
BufferNext ENDP

;  Inserisce nel buffer comandi CX caratteri di StrBuffer
BufferPutString PROC NEAR
           push si
           mov  di, [WritePtr]
           mov  [ReadPtr], di
@@Loop1:   SEGES
           lodsb
           call @@Put
           loop @@Loop1
           pop  si
           push di
           mov  al, EOS
@@Loop2:   call @@Put
           jne  @@Loop2
           pop  di
@@Put:     mov  [di], al
           call BufferNext
           mov  [WritePtr], di
           ret
BufferPutString ENDP

;  Copia in StrBuffer la stringa puntata da DI nel buffer comandi
BufferGetString PROC NEAR
           push si
           mov  [LastRead], di
@@Loop:    mov  al, [di]
           mov  es:[si], al
           inc  si
           call BufferNext
           cmp  al, EOS
           jne  @@Loop
           pop  si
           and  [Flags], NOT RESETREAD
           ret
BufferGetString ENDP

;  Legge la stringa precedente nel buffer comandi
BufferGetPrev PROC NEAR
           test [Flags], RESETREAD
           jnz  @@Read
           call @@Sub
           mov  [ReadPtr], di
@@Read:    call @@Sub
           jmp  BufferGetString
@@Sub:     call BufferPrev
           je   @@Sub
@@Sub1:    call BufferPrev
           jne  @@Sub1
           jmp  BufferNext
BufferGetPrev ENDP

;  Legge la stringa successiva nel buffer comandi
BufferGetNext PROC NEAR
           call BufferGetString
;  Subroutine!
SkipEOS:
           call BufferPrev
@@Loop:    call BufferNext
           je   @@Loop
@@Exit:    mov  [ReadPtr], di
           ret
BufferGetNext ENDP

;  Reinizializza il puntatore in lettura
;
BufferResetReadPtr PROC NEAR
           call BufferResetLastPtr
           call TestEmptyBuffer
           mov  di, [WritePtr]
           jmp  SkipEOS
BufferResetReadPtr ENDP

BufferDelEntry PROC NEAR
           call TestEmptyBuffer
           mov  di, [LastRead]
           or   di, di
           jnz  @@Entry
           ret
@@Loop:    call BufferNext
@@Entry:   mov  BYTE PTR [di], EOS
           jne  @@Loop
           mov  di, [ReadPtr]
           cmp  BYTE PTR [di], EOS
           jne  @@Read
           call BufferInitAll
           jmp  DeleteLine
@@Read:    jmp  DownKey
BufferDelEntry ENDP

;  Inizializza le variabili relative al buffer comandi
BufferInitAll PROC NEAR
           mov  di, [BuffStart]
           mov  [WritePtr], di
           mov  [ReadPtr], di
@@Loop:    mov  BYTE PTR [di], EOS
           inc  di
           cmp  di, [BuffEnd]
           jb   @@Loop
; Subroutine!
BufferResetLastPtr:
           mov  [LastRead], 0
           or   [Flags], RESETREAD
           ret
BufferInitAll ENDP

;  Routine di input bufferizzato
NewBufferedInput PROC NEAR
         IF IS8086
           push cx
           push dx
           push bx
           push bp
           push si
           push di
         ELSE
           pusha                            ; Salva i registri standard
         ENDIF
           push ds
           push es
;
           push ds
           pop  es
           mov  si, dx
           cmp  BYTE PTR [si], 0
           je   @@Exit                      ; Chiamata non valida
           dec  BYTE PTR [si]               ; Aggiustamento temporaneo lunghezza
           inc  si
           inc  si
           mov  al, [si]
           push cs
           pop  ds
           mov  [FirstChar], al
           call LineEdit                    ; Inserimento linea
           inc  BYTE PTR es:[si-2]          ; Ripristina l'esatto valore
           mov  dl, CR                      ; Stampa il CR finale
           MsDos 02h
;
@@Exit:    pop  es
           pop  ds
         IF IS8086
           pop  di
           pop  si
           pop  bp
           pop  bx
           pop  dx
           pop  cx
         ELSE
           popa
         ENDIF
           ret                              ; ... ed esce
NewBufferedInput ENDP

;  Nuovo interrupt 21h
NewDosInterrupt PROC FAR
           test cs:[Flags], ACTIVE          ; Siamo attivi?
           jz   @@Dos                       ; No, salta al DOS
           cmp  ah, 0Ah                     ; Funzione di input bufferizzato?
           je   @@Lab1                      ; Ok, intercetta
           cmp  ax, 0C0Ah                   ; Clear buffer e chiama funz. 0Ah?
           jne  @@Dos                       ; No, vai al DOS
;
           MsDos 0CFFh                      ; Pulisce il buffer di tastiera
           jmp  SHORT @@Lab2
;
@@Lab1:    or   cs:[Flags], PROMPT
           push bx
           push es
           MsDos 62h                        ; Get PSP of Current Program
           mov  es, bx
           cmp  bx, es:[16h]                ; PSP del programma precedente
           pop  es
           pop  bx
           je   @@Lab3
@@Lab2:    and  cs:[Flags], NOT PROMPT
@@Lab3:    mov  cs:[SavedSS], ss            ; Salva SS ed SP...
           mov  cs:[SavedSP], sp
           cli
           mov  ss, cs:[NewSS]              ; ... e li reinizializza
           mov  sp, OFFSET StackTop
           sti
           cld
           call NewBufferedInput            ; Ok, esegui la routine
           cli
           mov  ss, cs:[SavedSS]            ; Ripristina SS ed SP
           mov  sp, cs:[SavedSP]
           mov  ax, 0A00h
           iret
@@Dos:     jmp  cs:[OldDosInt]
NewDosInterrupt ENDP

EndOfTSR:

NotTSRMessage:
  DB  'XDLE non residente: comando ignorato',CR,LF,'$'
UninstMessage:
  DB  'XDLE rimosso $'
UninstSuccess:
  DB  '(memoria rilasciata al DOS)',CR,LF,'$'
UninstError:
  DB  '(memoria non recuperata)',CR,LF,'$'
ErrorMsg1  DB  'Parametro '
ErrorMsg2  DB  '  $'
ErrorMsg3  DB  '  errato (XDLE /? per la lista dei comandi)',CR,LF,'$'
OutRangeMessage DB  'Parametro /'
OutRangeMsg2    DB  '  fuori intervallo (XDLE /? per i valori consentiti)',CR,LF,'$'
PromptMessage:
  DB  'XDLE 1.5 - (c) 1990-92 Alessandro Scotti',CR,LF,'$'
HelpMessage:
  DB  CR,LF
  DB  'XDLE - Dos Line Editor 1.5 - Copyright (c) 1990-92 Alessandro Scotti',CR,LF
  DB  CR,LF
  DB  'Uso: XDLE [/opzione[/opzione...]] [/comando]',CR,LF
  DB  '     /ca     : attiva il programma',CR,LF
  DB  '     /cd     : disattiva il programma',CR,LF
  DB  '     /ci     : informazioni sulla copia residente',CR,LF
  DB  '     /cr     : rimuovi il programma dalla memoria',CR,LF
  DB  '     /bNUM   : cambia la dimensione del buffer (256-10240)',CR,LF
  DB  '     /i      : inizia in modo "insert"',CR,LF
  DB  '     /h, /?  : mostra la lista dei comandi',CR,LF
  DB  '     /mNUM   : salva solo le linee piu'' lunghe di NUM (0-127)',CR,LF
  DB  '     /n      : abilita/disabilita il cambiamento di dimensione del cursore',CR,LF
  DB  '     /o      : inizia in modo "overwrite"',CR,LF
  DB  '     /tNUM   : numero di spazi per il TAB (2-12)',CR,LF
  DB  '$'
ActiveMessage:
  DB  'XDLE attivato',CR,LF,'$'
DeactMessage:
  DB  'XDLE disattivato (XDLE /CA per riattivare)',CR,LF,'$'
VersionMessage:
  DB  "E' necessaria una versione DOS non inferione alla 3.0",CR,LF,'$'

DleTSRFlag     DB    0
BufferSize     DW    384                         ; Dimensione buffer comandi

S_BUFFER       EQU   'B'                         ; Opzioni command line
S_INS          EQU   'I'
S_MIN          EQU   'M'
S_OVR          EQU   'O'
S_TAB          EQU   'T'

TABMIN         EQU   2
TABMAX         EQU   12
BUFFERMIN      EQU   256
BUFFERMAX      EQU   10*1024                     ; 10k
LENMAX         EQU   127
TSRTESTSIZE    EQU   TSRTestEnd - TSRTestStart

PrintError PROC NEAR
           mov  dx, OFFSET ErrorMsg1
           call Print
           mov  dx, OFFSET ErrorMsg3
           cmp  BYTE PTR [ErrorMsg3], ' '
           jne  SHORT Print
           inc  dx
;  Subroutine!
Print:
           MsDos 09h
           stc
           ret
PrintError ENDP

;  Esamina la memoria alla ricerca di una copia residente del DLE (aggiorna ES)
InitTestTSR PROC NEAR
           mov  [DleTSRFlag], FALSE
           mov  si, OFFSET TSRTestStart     ; Inizio zona da confrontare
           mov  bx, cs:[02h]                ; Fine memoria
@@Loop:    or   bx, bx                      ; Primo segmento?
           je   @@Exit                      ; Si, fine ricerca
           mov  ax, [si]
           mov  di, si
           mov  es, bx
           cmp  ax, es:[di]                 ; Confronta una word
           je   @@TryThis                   ; Ok, prova a confrontare il resto
@@DecLoop: dec  bx                          ; Segmento precedente
           jmp  @@Loop
@@TryThis: mov  ax, cs
           cmp  bx, ax
           je   @@DecLoop                   ; Abbiamo trovato questa copia!
           cmp  es:[IDCode+0], 'D'          ; Controlla i codici di identita'
           jne  @@DecLoop
           cmp  es:[IDCode+1], 'L'
           jne  @@DecLoop
           cmp  es:[IDCode+2], 'E'
           jne  @@DecLoop
           push si                          ; Salva SI
           mov  cx, TSRTESTSIZE
           rep  cmpsb                       ; Controlla il programma
           pop  si                          ; Recupera SI
           jne  @@DecLoop                   ; Errore, cerca ancora
           mov  [DleTSRFlag], TRUE          ; Ok, trovato
           ret
@@Exit:    push ds                          ; Non trovato: ripristina ES
           pop  es
           ret
InitTestTSR ENDP

;  Controlla se DLE e' gia' installato
TestDleTSR PROC NEAR
           cmp  [DleTSRFlag], TRUE
           ret
TestDleTSR ENDP

;  Installa il programma
InstallDle PROC NEAR
           call TestDleTSR
           clc
           je   @@Exit                      ; Gia' istallato
           push es
           MsDos 3521h                      ; ES:BX = indirizzo interrupt 21h
           mov  [OldDosIntO], bx            ; Salva BX
           mov  [OldDosIntS], es            ; Salva ES
           pop  es
           mov  dx, OFFSET NewDosInterrupt
           MsDos 2521h                      ; Istalla il nuovo interrupt
           mov  [NewSS], cs                 ; Salva il CS
           mov  [IDCode+0], 'D'             ; Codice di controllo ('DLE')
           mov  [IDCode+1], 'L'
           mov  [IDCode+2], 'E'
           mov  [Flags], ACTIVE
           stc
@@Exit:    ret
InstallDle ENDP

;  Stampa AX in esadecimale
HexWriteAX PROC NEAR
           push ax
           xchg al, ah
           call @@WriteAL
           pop  ax
@@WriteAL: push ax
         IF IS8086
           mov  cl, 4
           ror  al, cl
         ELSE
           ror  al, 4
         ENDIF
           call @@Nibble
           pop  ax
@@Nibble:  and  al, 0Fh
           cmp  al, 9
           jb   @@Lab0
           add  al, 7
@@Lab0:    add  al, '0'
           mov  dl, al
           MsDos 02h
           ret
HexWriteAX ENDP

;  Stampa AX in esadecimale
DecWriteAX PROC NEAR
           mov   bx, '  '
           push  bx                         ; Usa lo stack come buffer
           push  bx
           push  bx
           mov   bp, sp
           mov   di, 5
           mov   bx, 10
           mov   [bp+di], bh                ; Fine stringa
@@Lab1:    xor   dx, dx
           div   bx
           add   dl, '0'
           dec   di
           mov   [bp+di], dl                ; Memorizza la cifra
           or    ax, ax
           jnz   @@Lab1
           xor   di, di
@@Lab2:    mov   al, [bp+di]                ; Cifra da stampare
           inc   di
           cmp   al, 0
           je    @@Exit
           cmp   al, ' '
           je    @@Lab2
           mov   dl, al
           MsDos 02h
           jmp   @@Lab2
@@Exit:    add   sp, 6                      ; Ripristina lo stack pointer
           ret
DecWriteAX ENDP

;  Settaggio dimensione buffer
SetBufferSize PROC NEAR
           call GetParmWord                 ; Leggi il numero
           jc   @@OutRange
           cmp  cx, BUFFERMIN               ; Controlla il range
           jb   @@OutRange
           cmp  cx, BUFFERMAX
           ja   @@OutRange
           mov  [BufferSize], cx
           clc
           ret
@@OutRange:mov  dx, OFFSET OutRangeMessage
           jmp  Print
SetBufferSize ENDP

;  Comandi speciali
CmdC PROC NEAR
           call TestDleTSR
           je   @@Lab0
           mov  dx, OFFSET NotTSRMessage
           jmp  Print
@@Lab0:    lodsb                            ; Successivo carattere
           call UpCase                      ; Trasforma in maiuscolo
           cmp  al, 32
           jae  @@OkPatch
           mov  al, '_'
@@OkPatch: mov  [ErrorMsg3], al
           cmp  al, 'A'                     ; A)ttiva
           je   @@ParmA
           cmp  al, 'D'                     ; D)isattiva
           je   @@ParmD
           cmp  al, 'I'                     ; I)nformazioni
           je   @@ParmI
         JUMPS
           cmp  al, 'R'                     ; R)imuovi
           je   @@ParmR
         NOJUMPS
           jmp  PrintError                  ; Errore
; Attiva il programma residente
@@ParmA:
           or   es:[Flags], ACTIVE
           mov  dx, OFFSET ActiveMessage
           jmp  Print
; Disattiva il programma residente
@@ParmD:
           and  es:[Flags], NOT ACTIVE
           mov  dx, OFFSET DeactMessage
           jmp  Print
; Informazioni sulla copia residente
@@ParmI:
           mov  dx, OFFSET InfoMsg1
           call Print
           mov  ax, es
           call HexWriteAX
           mov  dx, OFFSET InfoMsg2
           call Print
           mov  ax, es:[BuffEnd]
           test ax, 0Fh
           jz   @@WriteLen
           or   ax, 0Fh
           inc  ax
@@WriteLen:
           call DecWriteAX
           mov  dx, OFFSET InfoMsg3
           call Print
           mov  al, S_BUFFER
           call @@OutSlash
           mov  ax, es:[BuffEnd]
           sub  ax, es:[BuffStart]
           call DecWriteAX
           test es:[Flags], INSERT
           jnz  @@I0
           mov  al, S_INS
           call @@OutSlash
@@I0:      mov  al, S_MIN
           call @@OutSlash
           mov  ax, es:[MinLength]
           call DecWriteAX
           test es:[Flags], OVERWRITE
           jz   @@I1
           mov  al, S_OVR
           call @@OutSlash
@@I1:      mov  al, S_TAB
           call @@OutSlash
           mov  ax, es:[TabSize]
           call DecWriteAX
           mov  dx, OFFSET InfoMsg4
           call Print
           mov  dx, OFFSET InfoMsg5
           push dx
           call @@CheckVector
           pop  dx
           jne  @@I3
           add  dx, 4
@@I3:      jmp  Print
@@OutSlash:push ax
           mov  dl, ' '
           MsDos 02h
           mov  dl, '/'
           MsDos 02h
           pop  dx
           MsDos 02h
           ret
InfoMsg1  DB  CR,LF,'XDLE si trova al segmento $'
InfoMsg2  DB  'h (lunghezza: $'
InfoMsg3  DB  ' bytes)',CR,LF
          DB  'Configurazione corrente:$'
InfoMsg4  DB  CR,LF,'Memoria $'
InfoMsg5  DB  'non recuperabile con XDLE /CR',CR,LF,'$'
; Rimuove il programma residente
@@ParmR:
           mov  dx, OFFSET UninstMessage
           call Print
           push es
           push ds
           pop  es
           call @@Destroy                   ; Cancella QUESTO programma!
           pop  es
           call @@CheckVector
           jne  @@DontFree
           push ds
           mov  ds, es:[OldDosIntS]         ; Ripristina il vecchio interrupt
           mov  dx, es:[OldDosIntO]
           MsDos 2521h
           pop  ds
           call @@Destroy                   ; Cancella la copia residente
           push es
           mov  es, es:[2Ch]                ; Segmento environment
           MsDos 49h                        ; Libera la memoria
           pop  es
           jc   @@Error                     ; Errore
           MsDos 49h                        ; Rimuove il programma dalla memoria
           jc   @@Error                     ; Errore
           mov  dx, OFFSET UninstSuccess
           jmp  Print
@@DontFree:and  es:[Flags], NOT ACTIVE      ; Disattiva la copia residente...
           call @@Destroy                   ; ... e la rende inutilizzabile
@@Error:   mov  dx, OFFSET UninstError
           jmp  Print
;  Subroutine per cancellare il programma dalla memoria
@@Destroy: push cx
           push di
           mov  di, OFFSET TSRTestStart     ; Inizio codice
           mov  cx, TSRTESTSIZE             ; Numero bytes da cancellare
           mov  al, 0
           rep  stosb                       ; Cancella il programma al segm. ES
           pop  di
           pop  cx
           ret
;  Subroutine per controllare l'interrupt 21h
@@CheckVector:
           push es
           MsDos 3521h
           mov  ax, es                      ; AX:BX = interrupt 21h
           pop  es
           mov  cx, es
           cmp  ax, cx                      ; E' il nostro?
           jne  @@ChkExit
           cmp  bx, OFFSET NewDosInterrupt
@@ChkExit: ret
CmdC ENDP

;  Schermata di aiuto
CmdH PROC NEAR
           mov  dx,OFFSET HelpMessage
           jmp  Print
CmdH ENDP

;  Inizia l'editing in modo inserimento
CmdI PROC NEAR
          and  es:[Flags], NOT OVERWRITE
          ret
CmdI ENDP

;  Settaggio minima lunghezza stringa da salvare
SetMinLength PROC NEAR
           call GetParmWord                 ; Leggi il numero
           jc   @@OutRange                  ; Errore, esci
           cmp  cx, LENMAX                  ; Controllo range
           ja   @@OutRange
           mov  es:[MinLength], cx          ; Ok, salva il numero
           clc
           ret
@@OutRange:mov  dx, OFFSET OutRangeMessage
           jmp  Print
SetMinLength ENDP

;  Commuta il flag che attiva/disattiva la variazione in altezza del cursore
CmdN PROC NEAR
           xor  es:[Flags], NOCHGCRS
           ret
CmdN ENDP

;  Inizia l'editing in modo overwrite
CmdO PROC NEAR
          or   es:[Flags], OVERWRITE
          ret
CmdO ENDP

;  Settaggio spazi TAB
SetTabSize PROC NEAR
           call GetParmWord                 ; Leggi il numero
           jc   @@OutRange                  ; Errore, esci
           cmp  cx, TABMIN                  ; Controlla il range
           jb   @@OutRange
           cmp  cx, TABMAX
           ja   @@OutRange
           mov  es:[TabSize], cx            ; Ok, salva il numero
           clc
           ret
@@OutRange:mov  dx, OFFSET OutRangeMessage
           jmp  Print
SetTabSize ENDP

;  Legge una word dalla command line e la ritorna in CX (CF=1 in caso di errore)
GetParmWord PROC NEAR
           xor  cx, cx
           lodsb
           call @@Test                      ; Il carattere e' nel range?
           jc   @@Exit                      ; No, esci
@@Loop:    mov  ah, 0
           sub  al, '0'
         IF IS8086
           push ax
           mov  ax, 10
           cwd
           mul  cx
           pop  dx
         ELSE
           imul dx, cx, 10                  ; DX = BX*10
         ENDIF
           jc   @@Exit
           add  ax, dx
           mov  cx, ax                      ; Nuovo numero
           lodsb                            ; Carattere successivo
           call @@Test                      ; E' nel range?
           jnc  @@Loop                      ; Si, continua
           clc
           dec  si                          ; Rende il carattere disponibile
@@Exit:    ret
;  Controlla se AL e' nell'intervallo '0'-'9'
@@Test:    cmp  al, '9'+1
           cmc
           jc   @@TestExit
           cmp  al, '0'
@@TestExit:ret
GetParmWord ENDP

;  Tavola dei comandi disponibili e relative procedure
;
ParmTable      DB  S_BUFFER
               DW  SetBufferSize
               DB  'C'
               DW  CmdC
               DB  '?'
               DW  CmdH
               DB  'H'
               DW  CmdH
               DB  S_INS
               DW  CmdI
               DB  S_MIN
               DW  SetMinLength
               DB  'N'
               DW  CmdN
               DB  S_OVR
               DW  CmdO
               DB  S_TAB
               DW  SetTabSize
               DB  0

;  Trasforma in maiuscolo il carattere AL
UpCase PROC NEAR
           cmp  al, 'a'                     ; Minore di 'a'?
           jb   @@Exit                      ; Si, esci
           cmp  al, 'z'                     ; Maggiore di 'z'?
           ja   @@Exit                      ; Si, esci
           sub  al, 'a'-'A'                 ; Converti in maiuscolo
@@Exit:    ret
UpCase ENDP

;  Analizza la linea comandi
ParseCmdLine PROC NEAR
           mov  si, 81h                     ; Inizio command line
@@Loop1:   lodsb                            ; Legge un carattere
           mov  WORD PTR [ErrorMsg2], '$$'
           cmp  al, CR                      ; Fine riga?
           clc
           je   @@Exit                      ; Si, esci
           cmp  al, SPACE                   ; Salta i blanks
           je   @@Loop1
           cmp  al, TAB
           je   @@Loop1
           call UpCase
           cmp  al, 32
           jae  @@OkPatch
           mov  al, '_'
@@OkPatch: cmp  al, '/'                     ; Inizio comando?
           mov  [ErrorMsg2], al
           jne  @@Error                     ; No, errore
           lodsb                            ; Leggi il comando
           call UpCase
           cmp  al, 32
           jae  @@Ok
           mov  al, '_'
@@Ok:      mov  [ErrorMsg2+1], al
           mov  [OutRangeMsg2], al
           mov  bx, OFFSET ParmTable        ; Cerca il comando nella tavola
@@Loop2:   cmp  BYTE PTR [bx], 0            ; Fine tavola?
           je   @@Error                     ; Si, errore
           cmp  al, [bx]                    ; E' il comando cercato?
           je   @@Exec                      ; Si, esegui
           add  bx, 3                       ; No, esamina l'entrata seguente
           jmp  @@Loop2
@@Exec:    inc  bx
           call [bx]                        ; Esegui il comando
           jc   SHORT @@Exit                ; Esci se il carry e' settato
           jmp  @@Loop1                     ; Altrimenti continua il ciclo
@@Error:   call PrintError
@@Exit:    ret
ParseCmdLine ENDP

Start:
           cld
         IFE IS8086
           mov  ax, 0FFFFh                  ; Se compilato con istruzioni
           mov  cl, 33                      ; estese esegue un controllo
           shl  ax, cl                      ; sulla CPU
           jnz  CPUOk
; Nota: i processori 80186/286/386 eseguono CL MOD 32 prima di eseguire lo
; shift, quindi AX viene shiftato di un solo bit e diventa 0FFFEh, mentre
; l'8086 esegue l'istruzione per intero, azzerando AX.
           mov  dx, OFFSET BadCPUMsg
           MsDos 09h
           MsDos 4C01h
BadCPUMsg  DB "Questa versione di XDLE necessita almeno di un 80186",CR,LF,"$"
CPUOk:
         ENDIF
           MsDos 30h                        ; Legge la versione DOS
           cmp  al, 3
           jae  Continue
           mov  dx, OFFSET VersionMessage   ; Esce se inferiore a 3.0
           MsDos 09h
           MsDos 4C01h
Continue:
           call InitTestTSR                 ; Cerca un eventuale DLE gia' TSR
           call ParseCmdLine                ; Analizza la command line
           jc   Exit                        ; Errore
           call InstallDle                  ; Istalla i vettori
           jnc  Exit                        ; Gia' istallato, esci
           jmp  ExitTSR                     ; Istalla
Exit:      MsDos 4C00h                      ; Torna al DOS

END Entry
